#include <FastLED.h>
//
#define DATA_RIGHT_PIN 10      //This is the helmet's right side matrix
#define DATA_LEFT_PIN 9       //This is the helmet's left side matrix
#define NUM_LEDS    56        // The LED frames are 7 LEDs in 8 groups/segments 
#define BRIGHTNESS  32        //Brightness level of leds adjustable and not to burn out the LEDS.
#define LED_TYPE    WS2812B   //Type of addressable LED strip I am using
#define COLOR_ORDER GRB       //Led type as manufactured.

//
#define DATA_RIGHT_LOWER_PIN 12      //This is the helmet's right side matrix
#define DATA_LEFT_LOWER_PIN 11       //This is the helmet's left side matrix
#define LOWER_NUM_LEDS    20        // The LED frames are 7 LEDs in 8 groups/segments 
#define BRIGHTNESS  32 
CRGB led_array[NUM_LEDS];
#define UPDATES_PER_SECOND 100
//..........................................
long randLED1,randLED2,randLED3,randLED4,randLED5,randLED6,randLED7,randLED8;
//
const uint8_t kMatrixWidth = 7;          // break up LED string into a matrix 7 leds in width x 8 groups(or rows) high.
const uint8_t kMatrixHeight = 8;        // Width x height = total number of leds in your string (here it is 7x8 = 56).
const bool    kMatrixSerpentineLayout = true;   // True if Leds are wired in a serpintine assembled pattern, false if led strip is laid out as a linear orientation.
//
// Calculates position on string based on coordinates.  This sets up the serpentine / zig zag to work.
uint16_t XY( uint8_t x, uint8_t y)
//                               XY(x,y) takes x and y coordinates and returns an LED index number,
//                               for use like this:  leds[ XY(x,y) ] == CRGB::Red;
//                               No error checking is performed on the ranges of x and y.
{
  uint16_t i;
//                               This function will return the right 'led index number' for
//                                a given set of X and Y coordinates on your matrix.
//
  if( kMatrixSerpentineLayout == false) {
    i = (y * kMatrixWidth) + x;
  }
  if( kMatrixSerpentineLayout == true) {
    if( y & 0x01) {
      // Odd rows run backwards
      uint8_t reverseX = (kMatrixWidth - 1) - x;
      i = (y * kMatrixWidth) + reverseX;
    } else {
      // Even rows run forwards
      i = (y * kMatrixWidth) + x;
    }
  }
    return i;
}
CRGBPalette16 currentPalette;  // Calling up the library for pallette to be assigned to "current pallette"
TBlendType    currentBlending;  // Calling up the library for method for blending colors
//
//............... SET UP......................................... 
void setup() {
    delay( 3000 ); // power-up safety delay
    Serial.begin(9600);    // Open Serial port to help trouble shoot when a button is being pressed.

    // Calling up FastLed code to assign the pin to the LED string being used, the size etc....
    
    FastLED.addLeds<LED_TYPE, DATA_RIGHT_PIN, COLOR_ORDER>(led_array, NUM_LEDS).setCorrection( TypicalLEDStrip );
    FastLED.addLeds<LED_TYPE, DATA_LEFT_PIN, COLOR_ORDER>(led_array, NUM_LEDS).setCorrection( TypicalLEDStrip );
    FastLED.setBrightness(  BRIGHTNESS );
    // Set up the ear outputs....
      pinMode(8, OUTPUT);
  digitalWrite(8,LOW);
    pinMode(7, OUTPUT);
  digitalWrite(7,LOW);
    pinMode(6, OUTPUT);
  digitalWrite(6,LOW);
    pinMode(5, OUTPUT);
  digitalWrite(5,LOW);
    pinMode(4, OUTPUT);
  digitalWrite(4,LOW);
    pinMode(13, OUTPUT);
  digitalWrite(13,LOW);
    pinMode(14, OUTPUT);
  digitalWrite(14,LOW);
    pinMode(15, OUTPUT);
  digitalWrite(15,LOW);
    pinMode(16, OUTPUT);
  digitalWrite(16,LOW);
    pinMode(17, OUTPUT);
  digitalWrite(17,LOW);
    currentPalette = RainbowColors_p;  //Calls up the rainbow pallette already canned in the library.
    currentBlending = LINEARBLEND;     //Calls up the blending method.
}
// ___________________SUBROUTINE ANIMATIONS START HERE _________________________
//
//
void clear()                      //  Clears the entire display to OFF of "black."
{
    for (int i = 0; i < NUM_LEDS; i++)
    {
        led_array[i] = CRGB::Black;
    }
    FastLED.show();
}
//
//.....................................
void fadeall() { for(int i = 0; i < NUM_LEDS; i++) { led_array[i].nscale8(250); } }

void rainbowSnake()  // Works great (probably scolls backwards still but effect is ok.
{
for(int i=0; i<=4;i++)
{
  static uint8_t hue = 0;

  // First slide the led in one direction
  for(int i = 0; i < NUM_LEDS; i++)
    {
    led_array[i] = CHSV(hue++, 255, 255);

    FastLED.show(); 

   delay(30);
  }
//
// Now go in the other direction. 
delay (200);
// 
  for(int i = (NUM_LEDS)-1; i >= 0; i--)
    {
    led_array[i] = CHSV(hue++, 255, 255);
    FastLED.show();

    delay(30);
  }
}
}
//
//..............................................
void lightRow(int row, CRGB color)
{
    for (int i = 0; i < kMatrixWidth; i++)
    {
        led_array[XY(i, row)] = color;                
    }
    FastLED.show();
}

CRGB rainbow[9] = {CRGB::Red, CRGB::DarkMagenta, CRGB::Blue,
                   CRGB::DarkCyan, CRGB::DarkGreen, CRGB::Chartreuse,
                   CRGB::Yellow, CRGB::OrangeRed, CRGB::Red};

void lightRows(int rows, CRGB color)
{
    for (int i = 0; i < rows; i++)
    {
        lightRow(i, color);
    }
}
// ....................................
void fullColor(CRGB color)
{
    lightRows(kMatrixHeight, color);
}
//
//
//......................................
void fullonRainbow() // Works Great !!!`Scrolls in the correct Top to bottom direction!
{
  for(int i=0; i<=2; i++)
  {
    for (int i = 0; i <= kMatrixHeight; i++)
    {
        lightRow(i, rainbow[i]);
        delay(100);
    }
}
}
//
//.......................................
void rainbowDrop()   //BUG to Be Fixed > This is scrolling the wrong way from bottom to top instead it must drop from top down.
{
for(int i=0; i<=2;i++)
{
    for (int i = 0; i < kMatrixHeight; i++)
    {
        CRGB currentColor = rainbow[kMatrixHeight-i];
        
        int currentPosition = kMatrixHeight;
        int previousPosition = currentPosition + 1;

        while(currentPosition > i)
        {
            currentPosition = currentPosition - 1;
            previousPosition = currentPosition + 1;

            lightRow(currentPosition, currentColor);
            if (previousPosition <= (kMatrixHeight - 1))
            {
                lightRow(previousPosition, CRGB::Black);
            }
            FastLED.show();
            delay(100);          
        }
    }
//
// clear out
    for (int i = 0; i < kMatrixHeight; i++)
    {
        CRGB currentColor = rainbow[kMatrixHeight - i];
        
        int currentPosition = i;
        int previousPosition = currentPosition - 1;

        while(currentPosition >= -1)
        {
            currentPosition = currentPosition - 1;
            previousPosition = currentPosition + 1;

            if (currentPosition != -1)
            {
                lightRow(currentPosition, currentColor);
            }
            lightRow(previousPosition, CRGB::Black);

            FastLED.show();
            delay(100);          
        }
    }
}
}
int cycleColor = 0;
//
//..................................................
void rainbowWiper()    // BUG - Bascially Works but scrolls in the wrong direction,,, from bottom up, but needs to go top down.
{
 for (int i=0; i<=7;i++)
 {
    CRGB currentColor = rainbow[cycleColor+1];
    cycleColor++;

    if (cycleColor > 7)
    {
        cycleColor = 0;  
    }

    int currentPosition = kMatrixHeight;
    int previousPosition = currentPosition + 1;

    while(currentPosition > -1)
    {
        currentPosition = currentPosition - 1;
        previousPosition = currentPosition + 1;

        if (currentPosition > -1)
        {
            lightRow(currentPosition, currentColor);            
        }
        if (previousPosition <= (kMatrixHeight - 1))
        {
            lightRow(previousPosition, CRGB::Black);
        }
        FastLED.show();
        delay(100);          
    }
}
}
//
// .....................................................
void bounce()
{
  for (int i=0; i<=7; i++)
  {
    CRGB currentColor = rainbow[cycleColor];
    cycleColor++;

    if (cycleColor > 7)
    {
        cycleColor = 0;
    }

    int bounceDistance = 4;
    int bottomArray[] = {0, 1, 2, 3};
    int topArray[] = {kMatrixHeight - 1,  kMatrixHeight - 2, kMatrixHeight -3, kMatrixHeight -4};

    for (int i = 0; i <= bounceDistance; i++)
    {
        lightRow(topArray[i], currentColor);    
        lightRow(bottomArray[i], currentColor);
        
        FastLED.show();
        delay(100);       
    }

  for (int i = bounceDistance; i =0; i--)
    {
        lightRow(topArray[i], CRGB::Black);    
        lightRow(bottomArray[i], CRGB::Black);
        FastLED.show();
        delay(400);       
    }
     for (int i = 0; i <= bounceDistance; i++)
    {
        lightRow(topArray[i], CRGB::Black);    
        lightRow(bottomArray[i], CRGB::Black);
        
        FastLED.show();
        delay(100);  
    }  
}
}
//...................................................
void rainbowCrash()
{
  for(int i=0; i<=7; i++)
  {
    CRGB currentColor = rainbow[cycleColor];
    cycleColor++;

    if (cycleColor > 7)
    {
        cycleColor = 0;
    }

    int bounceDistance = 3;
    int bottomArray[] = {0, 1, 2, 3};
    int topArray[] = {kMatrixHeight - 1,  kMatrixHeight - 2, kMatrixHeight -3, kMatrixHeight -4};

    for (int i = 0; i <= bounceDistance; i++)
    {
        lightRow(topArray[i], currentColor);    
        lightRow(bottomArray[i], currentColor);
        FastLED.show();
        delay(200);
        lightRow(topArray[i], CRGB::Black);    
        lightRow(bottomArray[i], CRGB::Black);
        FastLED.show();
        delay(5); 

    }
        for (int i = 3; i >=0; i--)
    {
        lightRow(topArray[i], CRGB::Black);    
        lightRow(bottomArray[i], CRGB::Black);
        FastLED.show();
        delay(200);
        lightRow(topArray[i], currentColor);    
        lightRow(bottomArray[i], currentColor);
        FastLED.show();
        delay(5); 
    }
 }
}
//
void randomRainbow()                     
{
  for (int i=0; i<=8; i++)
  {
    for (int i = 0; i < NUM_LEDS; i++)
    {
        randLED1= random(1,56);
            led_array[randLED1] = CRGB::Red;
        randLED2= random(1,56);
            led_array[randLED2] = CRGB::OrangeRed;
        randLED3= random(1,56);
            led_array[randLED3] = CRGB::Yellow;
        randLED4= random(1,56);
            led_array[randLED4] = CRGB::Chartreuse;
        randLED5= random(1,56);
            led_array[randLED5] = CRGB::DarkGreen;
        randLED6= random(1,56);
            led_array[randLED6] = CRGB::DarkCyan;
        randLED7= random(1,56);
            led_array[randLED7] = CRGB::Blue;
        randLED8= random(1,56);
            led_array[randLED8] = CRGB::DarkMagenta;
        FastLED.show();
        delay(100);
        led_array[randLED1] = CRGB::Black;
        led_array[randLED2] = CRGB::Black;
        led_array[randLED3] = CRGB::Black;
        led_array[randLED4] = CRGB::Black;
        led_array[randLED5] = CRGB::Black;
        led_array[randLED6] = CRGB::Black;
        led_array[randLED7] = CRGB::Black;
        led_array[randLED8] = CRGB::Black;
        
    }
    FastLED.show();
}
}
//
// ------------------------------------ LOOP PROGRAM BELOW ------------------------
void loop()
{
  // \/\/___Code for the Heartbeat signal on the earcups____\/\/
//         Drives both right and left ears....
for( int i=4; i<=8;i++)
{
  digitalWrite(i, HIGH); // turn the Green LED ON 
    delay(200);
  digitalWrite(i, LOW); // turn the Green LED OFF
}
for( int i=13; i<=17;i++)
{
  digitalWrite(i, HIGH); // turn the Green LED ON 
    delay(200);
  digitalWrite(i, LOW); // turn the Green LED OFF
}
// _________________Code for the White Bar Lights _____________
/*for(int i=48; i<=54; i+2)
{
    digitalWrite(i, HIGH); // turn the White LED ON Moving right
    delay(200);
      digitalWrite(i, LOW); // turn the White LED OFF
}
for(int i=54; i<=48; i-2)
{
    digitalWrite(i, HIGH); // turn the White LED ON Moving left
    delay(200);
      digitalWrite(i, LOW); // turn the White LED OFF
}
//
for(int i=31; i<=37; i+2)
{
    digitalWrite(i, HIGH); // turn the Left most Green LED ON 
    delay(200);
      digitalWrite(i, LOW); // turn the Left most Green LED OFF
}
for(int i=37; i<=31; i-2)
{
    digitalWrite(i, HIGH); // turn the White LED ON Moving left
    delay(200);
      digitalWrite(i, LOW); // turn the White LED OFF
      */
     rainbowDrop();
delay (2000);
      rainbowWiper();
delay (2000);
      rainbowSnake();
delay (2000);
      rainbowCrash();
delay (2000);
      bounce();
delay (2000);
      randomRainbow();
delay (4000);
FastLED.clear();
FastLED.show();
FastLED.delay(1000 / UPDATES_PER_SECOND);
}
